@@ -1,38 +1,15 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt for Python.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "abstractmetatype.h"
 #include "abstractmetabuilder.h"
 #include "abstractmetalang.h"
 #include "messages.h"
 #include "typedatabase.h"
-#include "typesystem.h"
-#include "parser/codemodel.h"
+#include "containertypeentry.h"
+
+#include "qtcompat.h"
+#include "typeinfo.h"
 
 #ifndef QT_NO_DEBUG_STREAM
 #  include <QtCore/QDebug>
@@ -43,23 +20,25 @@
 #include <QtCore/QSharedPointer>
 #include <QtCore/QStack>
 
+using namespace Qt::StringLiterals;
+
 using AbstractMetaTypeCPtr = QSharedPointer<const AbstractMetaType>;
 
 const QSet<QString> &AbstractMetaType::cppFloatTypes()
 {
-    static const QSet<QString> result{u"double"_qs, u"float"_qs};
+    static const QSet<QString> result{u"double"_s, u"float"_s};
     return result;
 }
 
 const QSet<QString> &AbstractMetaType::cppSignedCharTypes()
 {
-    static const QSet<QString> result{u"char"_qs, u"signed char"_qs};
+    static const QSet<QString> result{u"char"_s, u"signed char"_s};
     return result;
 }
 
 const QSet<QString> &AbstractMetaType::cppUnsignedCharTypes()
 {
-    static const QSet<QString> result{u"unsigned char"_qs};
+    static const QSet<QString> result{u"unsigned char"_s};
     return result;
 }
 
@@ -73,14 +52,14 @@ const QSet<QString> &AbstractMetaType::cppSignedIntTypes()
 {
     static QSet<QString> result;
     if (result.isEmpty()) {
-        result = {u"char"_qs, u"signed char"_qs, u"short"_qs, u"short int"_qs,
-                  u"signed short"_qs, u"signed short int"_qs,
-                  u"int"_qs, u"signed int"_qs,
-                  u"long"_qs, u"long int"_qs,
-                  u"signed long"_qs, u"signed long int"_qs,
-                  u"long long"_qs, u"long long int"_qs,
-                  u"signed long long int"_qs,
-                  u"ptrdiff_t"_qs};
+        result = {u"char"_s, u"signed char"_s, u"short"_s, u"short int"_s,
+                  u"signed short"_s, u"signed short int"_s,
+                  u"int"_s, u"signed int"_s,
+                  u"long"_s, u"long int"_s,
+                  u"signed long"_s, u"signed long int"_s,
+                  u"long long"_s, u"long long int"_s,
+                  u"signed long long int"_s,
+                  u"ptrdiff_t"_s};
         result |= cppSignedCharTypes();
     }
     return result;
@@ -90,12 +69,12 @@ const QSet<QString> &AbstractMetaType::cppUnsignedIntTypes()
 {
     static QSet<QString> result;
     if (result.isEmpty()) {
-        result = {u"unsigned short"_qs, u"unsigned short int"_qs,
-                  u"unsigned"_qs, u"unsigned int"_qs,
-                  u"unsigned long"_qs, u"unsigned long int"_qs,
-                  u"unsigned long long"_qs,
-                  u"unsigned long long int"_qs,
-                  u"size_t"_qs};
+        result = {u"unsigned short"_s, u"unsigned short int"_s,
+                  u"unsigned"_s, u"unsigned int"_s,
+                  u"unsigned long"_s, u"unsigned long int"_s,
+                  u"unsigned long long"_s,
+                  u"unsigned long long int"_s,
+                  u"size_t"_s};
         result |= cppUnsignedCharTypes();
     }
     return result;
@@ -107,7 +86,7 @@ const QSet<QString> &AbstractMetaType::cppIntegralTypes()
     if (result.isEmpty()) {
         result |= cppSignedIntTypes();
         result |= cppUnsignedIntTypes();
-        result.insert(u"bool"_qs);
+        result.insert(u"bool"_s);
     }
     return result;
 }
@@ -118,7 +97,7 @@ const QSet<QString> &AbstractMetaType::cppPrimitiveTypes()
     if (result.isEmpty()) {
         result |= cppIntegralTypes();
         result |= cppFloatTypes();
-        result.insert(u"wchar_t"_qs);
+        result.insert(u"wchar_t"_s);
     }
     return result;
 }
@@ -245,7 +224,7 @@ bool AbstractMetaType::applyArrayModification(QString *errorMessage)
 {
 
     if (d->m_pattern == AbstractMetaType::NativePointerAsArrayPattern) {
-        *errorMessage = QLatin1String("<array> modification already applied.");
+        *errorMessage = u"<array> modification already applied."_s;
         return false;
     }
     if (!d->m_arrayElementType.isNull())  {
@@ -555,10 +534,10 @@ bool AbstractMetaType::hasTemplateChildren() const
 static inline QString formatArraySize(int e)
 {
     QString result;
-    result += QLatin1Char('[');
+    result += u'[';
     if (e >= 0)
         result += QString::number(e);
-    result += QLatin1Char(']');
+    result += u']';
     return result;
 }
 
@@ -566,13 +545,13 @@ QString AbstractMetaTypeData::formatSignature(bool minimal) const
 {
     QString result;
     if (m_constant)
-        result += QLatin1String("const ");
+        result += u"const "_s;
     if (m_volatile)
-        result += QLatin1String("volatile ");
+        result += u"volatile "_s;
     if (m_pattern == AbstractMetaType::ArrayPattern) {
         // Build nested array dimensions a[2][3] in correct order
         result += m_arrayElementType->minimalSignature();
-        const int arrayPos = result.indexOf(QLatin1Char('['));
+        const int arrayPos = result.indexOf(u'[');
         if (arrayPos != -1)
             result.insert(arrayPos, formatArraySize(m_arrayElementCount));
         else
@@ -581,29 +560,29 @@ QString AbstractMetaTypeData::formatSignature(bool minimal) const
         result += m_typeEntry->qualifiedCppName();
     }
     if (!m_instantiations.isEmpty()) {
-        result += QLatin1Char('<');
+        result += u'<';
         if (minimal)
-            result += QLatin1Char(' ');
+            result += u' ';
         for (int i = 0, size = m_instantiations.size(); i < size; ++i) {
             if (i > 0)
-                result += QLatin1Char(',');
+                result += u',';
             result += m_instantiations.at(i).minimalSignature();
         }
-        result += QLatin1String(" >");
+        result += u" >"_s;
     }
 
     if (!minimal && (!m_indirections.isEmpty() || m_referenceType != NoReference))
-        result += QLatin1Char(' ');
+        result += u' ';
     for (Indirection i : m_indirections)
         result += TypeInfo::indirectionKeyword(i);
     switch (m_referenceType) {
     case NoReference:
         break;
     case LValueReference:
-        result += QLatin1Char('&');
+        result += u'&';
         break;
     case RValueReference:
-        result += QLatin1String("&&");
+        result += u"&&"_s;
         break;
     }
     return result;
@@ -629,7 +608,7 @@ QString AbstractMetaTypeData::formatPythonSignature() const
      */
     QString result;
     if (m_pattern == AbstractMetaType::NativePointerAsArrayPattern)
-        result += QLatin1String("array ");
+        result += u"array "_s;
     // We no longer use the "const" qualifier for heuristics. Instead,
     // NativePointerAsArrayPattern indicates when we have <array> in XML.
     // if (m_typeEntry->isPrimitive() && isConstant())
@@ -637,12 +616,12 @@ QString AbstractMetaTypeData::formatPythonSignature() const
     if (!m_typeEntry->isPrimitive() && !m_typeEntry->isSmartPointer()) {
         const QString package = m_typeEntry->targetLangPackage();
         if (!package.isEmpty())
-            result += package + QLatin1Char('.');
+            result += package + u'.';
     }
     if (m_pattern == AbstractMetaType::ArrayPattern) {
         // Build nested array dimensions a[2][3] in correct order
         result += m_arrayElementType->formatPythonSignature();
-        const int arrayPos = result.indexOf(QLatin1Char('['));
+        const int arrayPos = result.indexOf(u'[');
         if (arrayPos != -1)
             result.insert(arrayPos, formatArraySize(m_arrayElementCount));
         else
@@ -651,13 +630,13 @@ QString AbstractMetaTypeData::formatPythonSignature() const
         result += m_typeEntry->targetLangName();
     }
     if (!m_instantiations.isEmpty()) {
-        result += QLatin1Char('[');
+        result += u'[';
         for (int i = 0, size = m_instantiations.size(); i < size; ++i) {
             if (i > 0)
-                result += QLatin1String(", ");
+                result += u", "_s;
             result += m_instantiations.at(i).formatPythonSignature();
         }
-        result += QLatin1Char(']');
+        result += u']';
     }
     if (m_typeEntry->isPrimitive())
         for (Indirection i : m_indirections)
@@ -666,7 +645,7 @@ QString AbstractMetaTypeData::formatPythonSignature() const
     // "PySide6.QtCore.Qt.ItemFlags" instead of "PySide6.QtCore.QFlags<Qt.ItemFlag>"
     if (m_typeEntry->isFlags())
         result = m_typeEntry->qualifiedTargetLangName();
-    result.replace(QLatin1String("::"), QLatin1String("."));
+    result.replace(u"::"_s, u"."_s);
     return result;
 }
 
@@ -763,7 +742,7 @@ AbstractMetaType AbstractMetaType::createVoid()
 {
     static QScopedPointer<AbstractMetaType> metaType;
     if (metaType.isNull()) {
-        static const TypeEntry *voidTypeEntry = TypeDatabase::instance()->findType(QLatin1String("void"));
+        static const TypeEntry *voidTypeEntry = TypeDatabase::instance()->findType(u"void"_s);
         Q_ASSERT(voidTypeEntry);
         metaType.reset(new AbstractMetaType(voidTypeEntry));
         metaType->decideUsagePattern();
@@ -773,17 +752,22 @@ AbstractMetaType AbstractMetaType::createVoid()
 
 void AbstractMetaType::dereference(QString *type)
 {
-    type->prepend(u"(*"_qs);
+    type->prepend(u"(*"_s);
     type->append(u')');
 }
 
+QString AbstractMetaType::dereferencePrefix(qsizetype n)
+{
+    const QChar c = n > 0 ? u'*' : u'&';
+    return QString(qAbs(n), c);
+}
+
 void AbstractMetaType::applyDereference(QString *type, qsizetype n)
 {
     if (n == 0)
         return;
 
-    const char c = n > 0 ? '*' : '&';
-    type->prepend(QString(qAbs(n), QLatin1Char(c)));
+    type->prepend(dereferencePrefix(n));
     type->prepend(u'(');
     type->append(u')');
 }
@@ -810,6 +794,11 @@ bool AbstractMetaType::isObjectType() const
     return d->m_typeEntry->isObject();
 }
 
+bool AbstractMetaType::isUniquePointer() const
+{
+    return isSmartPointer() && d->m_typeEntry->isUniquePointer();
+}
+
 bool AbstractMetaType::isPointer() const
 {
     return !d->m_indirections.isEmpty()
@@ -826,14 +815,14 @@ bool AbstractMetaType::isCString() const
 {
     return isNativePointer()
         && d->m_indirections.size() == 1
-        && name() == QLatin1String("char");
+        && name() == u"char";
 }
 
 bool AbstractMetaType::isVoidPointer() const
 {
     return isNativePointer()
         && d->m_indirections.size() == 1
-        && name() == QLatin1String("void");
+        && name() == u"void";
 }
 
 bool AbstractMetaType::isUserPrimitive() const
@@ -863,25 +852,6 @@ bool AbstractMetaType::isWrapperPassedByReference() const
            && !isPointer();
 }
 
-qsizetype AbstractMetaType::shouldDereferenceArgument() const
-{
-    if (isWrapperPassedByReference() || valueTypeWithCopyConstructorOnlyPassed()
-        || isObjectTypeUsedAsValueType()) {
-        return 1;
-    }
-
-    if (!d->m_typeEntry->isContainer())
-        return 0;
-
-    qsizetype result = -d->m_indirections.size();
-
-    // For opaque containers, the cppArg in the generated code is a pointer
-    if (generateOpaqueContainer())
-        ++result;
-
-    return result;
-}
-
 bool AbstractMetaType::isCppIntegralPrimitive() const
 {
     return  d->m_typeEntry->isCppIntegralPrimitive();
@@ -920,7 +890,7 @@ std::optional<AbstractMetaType>
 AbstractMetaType::fromString(QString typeSignature, QString *errorMessage)
 {
     typeSignature = typeSignature.trimmed();
-    if (typeSignature.startsWith(QLatin1String("::")))
+    if (typeSignature.startsWith(u"::"))
         typeSignature.remove(0, 2);
 
     auto &cache = *metaTypeFromStringCache();
@@ -941,7 +911,7 @@ AbstractMetaType::fromString(QString typeSignature, QString *errorMessage)
 AbstractMetaType AbstractMetaType::fromTypeEntry(const TypeEntry *typeEntry)
 {
     QString typeName = typeEntry->qualifiedCppName();
-    if (typeName.startsWith(QLatin1String("::")))
+    if (typeName.startsWith(u"::"))
         typeName.remove(0, 2);
     auto &cache  = *metaTypeFromStringCache();
     auto it = cache.find(typeName);
